<?php

namespace CL\Daos;

use CL\Models\Assignment;
use CL\Models\CourseSection;
use CL\Models\User;

class CourseSyncDao
{

    private $clDao;

    public function __construct()
    {
        $this->clDao = new CampusLabsDao();
    }
    /**
     * @param int $roleId
     * @param int $courseId
     * @return \moodle_recordset
     */
    private function getUsersByRoleIdForCourse($roleId, $courseId)
    {
        global $DB;
        $sql='SELECT u.id AS id, firstname, username, idnumber, lastname, email, ue.status as suspended
        FROM {role_assignments} ra
            INNER JOIN {context} c ON ra.contextid = c.id
            INNER JOIN {user} u ON ra.userid = u.id
            INNER JOIN {user_enrolments} ue ON ue.userid = ra.userid
            INNER JOIN {enrol} e ON ue.enrolid = e.id AND e.courseid = c.instanceid
        WHERE e.courseid = ? AND ra.roleid = ?;';
        return $DB->get_recordset_sql($sql, array($courseId, $roleId));
    }

    /**
     * @param int $courseId
     * @return \moodle_recordset
     */
    private function getAssignments($courseId)
    {
        global $DB;
        return $DB->get_recordset_sql("SELECT * FROM {assign} WHERE course=?", array($courseId));
    }

    /**
     * @param $assignModuleId
     * @param $assignmentId
     * @param $courseId
     * @return string
     */
    private function getScoringMethod($assignModuleId, $assignmentId, $courseId) {
        global $DB;

        //get assignment module id
        $sql = 'SELECT id from {course_modules} WHERE course=? AND module=? AND instance=?';
        $assignmentModule = $DB->get_record_sql($sql, array($courseId, $assignModuleId, $assignmentId));


        $sql = 'SELECT activemethod FROM {grading_areas} WHERE contextid=? AND component=? AND areaname=?';
        $gradingAreaActiveMethod = $DB->get_record_sql($sql, array(\context_module::instance($assignmentModule->id)->id, 'mod_assign', 'submissions'));

        return $this->formatScoringMethod($gradingAreaActiveMethod->activemethod);

    }

    /**
     * @param string $gradingAreaActiveMethod
     * @return string
     */
    private function formatScoringMethod($gradingAreaActiveMethod)
    {
        if ($gradingAreaActiveMethod === 'rubric') {
            $scoringMethod = 'Rubric';
        }
        else if ($gradingAreaActiveMethod === 'guide') {
            $scoringMethod = 'Marking guide';
        }
        else {
            $scoringMethod = 'Simple direct grading';
        }

        return $scoringMethod;
    }

    /**
     * @param $courseId
     * @param $DB
     * @return mixed
     */
    private function getCourseSection($courseId)
    {
        global $DB;
        $course = $DB->get_record_sql("SELECT * FROM {course} WHERE id=?", array($courseId));
        return $course;
    }

    /**
     * @param int $courseId
     * @return CourseSection
     */
    private function loadCourse($courseId)
    {
        global $CFG,
               $DB;
        $course = $this->getCourseSection($courseId);
        $campusLabsCourse = new CourseSection($course->fullname, $course->id, strip_tags($course->summary), $this->formatDate($course->startdate), null);

        //get id for assignment module
        $assignModule = $DB->get_record('modules', array('name' => 'assign'), 'id');

        //assignments
        foreach ($this->getAssignments($courseId) as $assignment) {
            $campusLabsCourse->addAssignment(
                new Assignment(
                    $assignment->id,
                    $CFG->wwwroot,
                    $assignment->name,
                    $this->formatAssignmentType($assignment),
                    strip_tags($assignment->intro),
                    $this->getScoringMethod($assignModule->id, $assignment->id, $courseId),
                    $assignment->grade,
                    $this->formatAttemptsAllowed($assignment->attemptreopenmethod),
                    (int)$assignment->maxattempts == -1 ? null : (int)$assignment->maxattempts,
                    ($assignment->allowsubmissionsfromdate == 0) ? null : $this->formatDate($assignment->allowsubmissionsfromdate),
                    ($assignment->cutoffdate == 0) ? null : $this->formatDate($assignment->cutoffdate),
                    ($assignment->duedate == 0) ? null : $this->formatDate($assignment->duedate)
                )
            );
        }

        //enrollments
        foreach ($this->getUsersByRoleIdForCourse(5, $courseId) as $student) {
            $campusLabsCourse->addEnrollment(new User($student->id, $this->getCampusIdentifier($student), $student->firstname, $student->lastname, $student->email, $student->suspended));
        }

        //instructors
        foreach ($this->getUsersByRoleIdForCourse(3, $courseId) as $instructor) {
            $campusLabsCourse->addInstructor(new User($instructor->id, $this->getCampusIdentifier($instructor), $instructor->firstname, $instructor->lastname, $instructor->email, $instructor->suspended));
        }

        return $campusLabsCourse;
    }

    /**
     * @param $user
     * @return string
     */
    private function getCampusIdentifier($user)
    {
        $campusIdentifier = \get_config('campuslabs_course_sync', 'identifier');

        
        switch($campusIdentifier)
        {
        	case 'id_number':
        		return $user->idnumber;
        	case 'username':
        		return $user->username;
        	case 'email':
        		return $user->email;
        	default:
        		return $user->id;
        		
        }
     }

    /**
     * @param string $assignmentAttempt
     * @return string
     */
    private function formatAttemptsAllowed($assignmentAttempt)
    {
        if ($assignmentAttempt === "none") {
            $attemptsAllowed = "Never";
        } else if ($assignmentAttempt === "manual") {
            $attemptsAllowed = "Manually";
        } else {
            $attemptsAllowed = "Automatically until pass";
        }
        return $attemptsAllowed;
    }

    /**
     * @param int $assignment
     * @return string
     */
    private function formatAssignmentType($assignment)
    {
        if ($assignment->nosubmissions === 1) {
            $assignmentType = "No Submission";
        } else {
            $assignmentType = "Online text or file submission";
        }
        return $assignmentType;
    }

    /**
     * @param int $epochTime
     * @return string
     */
    private function formatDate($epochTime)
    {
        return (new \DateTime("@$epochTime"))->format('c');
    }

    /**
     * @param $courseId
     * @return string
     */
    public function sendToCampusLabs($courseId)
    {
        $course = $this->loadCourse($courseId);
        return $this->clDao->sendCourseSectionToCampusLabs($course);
    }

}